@@ -9,6 +9,8 @@
  * notice and this notice are preserved.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 
 #include <telepathy-glib/telepathy-glib.h>
@@ -20,20 +22,23 @@ typedef struct {
 } InspectChannelData;
 
 static void
-channel_ready_cb (TpChannel *channel,
-    const GError *error,
+channel_ready_cb (GObject *source,
+    GAsyncResult *result,
     gpointer user_data)
 {
+  TpChannel *channel = TP_CHANNEL (source);
   InspectChannelData *data = user_data;
   guint handle_type, handle;
   gchar *channel_type;
   gchar **interfaces, **iter;
+  GError *error = NULL;
 
-  if (error != NULL)
+  if (!tp_proxy_prepare_finish (channel, result, &error))
     {
       g_warning ("%s", error->message);
       data->exit_status = 1;
       g_main_loop_quit (data->main_loop);
+      g_clear_error (&error);
       return;
     }
 
@@ -78,19 +83,22 @@ channel_ready_cb (TpChannel *channel,
 }
 
 static void
-connection_ready_cb (TpConnection *connection,
-    const GError *ready_error,
+connection_ready_cb (GObject *source,
+    GAsyncResult *result,
     gpointer user_data)
 {
   InspectChannelData *data = user_data;
   GError *error = NULL;
+  TpConnection *connection = TP_CONNECTION (source);
   TpChannel *channel = NULL;
 
-  if (ready_error != NULL)
+
+  if (!tp_proxy_prepare_finish (connection, result, &error))
     {
-      g_warning ("%s", ready_error->message);
+      g_warning ("%s", error->message);
       data->exit_status = 1;
       g_main_loop_quit (data->main_loop);
+      g_clear_error (&error);
       return;
     }
 
@@ -106,7 +114,7 @@ connection_ready_cb (TpConnection *connection,
       return;
     }
 
-  tp_channel_call_when_ready (channel, channel_ready_cb, data);
+  tp_proxy_prepare_async (channel, NULL, channel_ready_cb, user_data);
 
   /* the channel will remain referenced as long as it has calls pending on
    * it */
@@ -170,7 +178,7 @@ main (int argc,
    * else has called (or will call) Connect(), so we won't call Connect()
    * on it ourselves
    */
-  tp_connection_call_when_ready (connection, connection_ready_cb, &data);
+  tp_proxy_prepare_async (connection, NULL, connection_ready_cb, &data);
 
   g_main_loop_run (data.main_loop);
 
