@@ -9,6 +9,8 @@
  * notice and this notice are preserved.
  */
 
+#include "config.h"
+
 #include "conn.h"
 
 #include <dbus/dbus-glib.h>
@@ -433,8 +435,7 @@ example_contact_list_connection_class_init (
 
   param_spec = g_param_spec_string ("account", "Account name",
       "The username of this user", NULL,
-      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB);
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_ACCOUNT, param_spec);
 
   param_spec = g_param_spec_uint ("simulation-delay", "Simulation delay",
