@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
+
 #include "telepathy-glib/account-manager-internal.h"
 #include "telepathy-glib/account-internal.h"
 
@@ -30,7 +32,6 @@
 #include <telepathy-glib/util.h>
 
 #include "telepathy-glib/account-manager.h"
-#include "telepathy-glib/_gen/signals-marshal.h"
 
 #define DEBUG_FLAG TP_DEBUG_ACCOUNTS
 #include "telepathy-glib/dbus-internal.h"
@@ -327,6 +328,7 @@ _tp_account_manager_update_most_available_presence (TpAccountManager *manager)
   TpAccount *account = NULL;
   GHashTableIter iter;
   gpointer value;
+  TpAccount *has_unset_presence = NULL;
 
   /* this presence is equal to the presence of the account with the
    * highest availability */
@@ -339,6 +341,14 @@ _tp_account_manager_update_most_available_presence (TpAccountManager *manager)
 
       p = tp_account_get_current_presence (a, NULL, NULL);
 
+      if (p == TP_CONNECTION_PRESENCE_TYPE_UNSET)
+        {
+          has_unset_presence = a;
+          /* There is no point comparing the presence as UNSET is the
+           * 'smallest' presence of all */
+          continue;
+        }
+
       if (tp_connection_presence_type_cmp_availability (p, presence) > 0)
         {
           account = a;
@@ -346,15 +356,23 @@ _tp_account_manager_update_most_available_presence (TpAccountManager *manager)
         }
     }
 
+  if (presence == TP_CONNECTION_PRESENCE_TYPE_OFFLINE &&
+      has_unset_presence != NULL)
+    {
+      /* Use an account having UNSET as presence as the 'best' one,
+       * see tp_account_manager_get_most_available_presence() */
+      account = has_unset_presence;
+    }
+
   priv->most_available_account = account;
   g_free (priv->most_available_status);
   g_free (priv->most_available_status_message);
 
   if (account == NULL)
     {
-      priv->most_available_presence = presence;
-      priv->most_available_status = NULL;
-      priv->most_available_status_message = NULL;
+      priv->most_available_presence = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
+      priv->most_available_status = g_strdup ("offline");
+      priv->most_available_status_message = g_strdup ("");
       return;
     }
 
@@ -581,8 +599,7 @@ tp_account_manager_class_init (TpAccountManagerClass *klass)
       G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       0,
-      NULL, NULL,
-      _tp_marshal_VOID__OBJECT_BOOLEAN,
+      NULL, NULL, NULL,
       G_TYPE_NONE,
       2, TP_TYPE_ACCOUNT, G_TYPE_BOOLEAN);
 
@@ -599,8 +616,7 @@ tp_account_manager_class_init (TpAccountManagerClass *klass)
       G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       0,
-      NULL, NULL,
-      g_cclosure_marshal_VOID__OBJECT,
+      NULL, NULL, NULL,
       G_TYPE_NONE,
       1, TP_TYPE_ACCOUNT);
 
@@ -621,8 +637,7 @@ tp_account_manager_class_init (TpAccountManagerClass *klass)
       G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       0,
-      NULL, NULL,
-      g_cclosure_marshal_VOID__OBJECT,
+      NULL, NULL, NULL,
       G_TYPE_NONE,
       1, TP_TYPE_ACCOUNT);
 
@@ -639,8 +654,7 @@ tp_account_manager_class_init (TpAccountManagerClass *klass)
       G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       0,
-      NULL, NULL,
-      g_cclosure_marshal_VOID__OBJECT,
+      NULL, NULL, NULL,
       G_TYPE_NONE,
       1, TP_TYPE_ACCOUNT);
 
@@ -660,8 +674,7 @@ tp_account_manager_class_init (TpAccountManagerClass *klass)
         G_TYPE_FROM_CLASS (klass),
         G_SIGNAL_RUN_LAST,
         0,
-        NULL, NULL,
-        _tp_marshal_VOID__UINT_STRING_STRING,
+        NULL, NULL, NULL,
         G_TYPE_NONE,
         3, G_TYPE_UINT, /* Presence type */
         G_TYPE_STRING,  /* status */
@@ -865,6 +878,9 @@ _tp_account_manager_account_presence_changed_cb (TpAccount *account,
 {
   TpAccountManager *manager = TP_ACCOUNT_MANAGER (user_data);
   TpAccountManagerPrivate *priv = manager->priv;
+  TpConnectionPresenceType p;
+  gchar *s;
+  gchar *msg;
 
   if (tp_connection_presence_type_cmp_availability (presence,
           priv->most_available_presence) > 0)
@@ -888,10 +904,19 @@ _tp_account_manager_account_presence_changed_cb (TpAccount *account,
     }
 
   return;
+
 signal:
+  /* Use tp_account_manager_get_most_available_presence() as the effective
+   * most available presence may differ of the one stored in
+   * priv->most_available_presence. */
+  p = tp_account_manager_get_most_available_presence (manager,
+      &s, &msg);
+
   g_signal_emit (manager, signals[MOST_AVAILABLE_PRESENCE_CHANGED], 0,
-      priv->most_available_presence, priv->most_available_status,
-      priv->most_available_status_message);
+      p, s, msg);
+
+  g_free (s);
+  g_free (msg);
 }
 
 static void
@@ -1127,6 +1152,10 @@ tp_account_manager_set_all_requested_presences (TpAccountManager *manager,
  * If no accounts are enabled or valid the output will be
  * (%TP_CONNECTION_PRESENCE_TYPE_OFFLINE, "offline", "").
  *
+ * Since 0.17.5, if the only connected accounts does not implement
+ * %TP_IFACE_CONNECTION_INTERFACE_SIMPLE_PRESENCE, the output will be
+ * (%TP_CONNECTION_PRESENCE_TYPE_AVAILABLE, "available", "").
+ *
  * The return value of this function is not guaranteed to have been retrieved
  * until tp_proxy_prepare_async() has finished; until then, the
  * value will be the same as if no accounts are enabled or valid.
@@ -1148,6 +1177,20 @@ tp_account_manager_get_most_available_presence (TpAccountManager *manager,
 
   priv = manager->priv;
 
+  if (priv->most_available_presence == TP_CONNECTION_PRESENCE_TYPE_UNSET)
+    {
+      /* The best we have is an account having UNSET as its presence, which
+       * means it's connected but does not implement SimplePresence; pretend
+       * we are available. */
+      if (status != NULL)
+        *status = g_strdup ("available");
+
+      if (message != NULL)
+        *message = g_strdup ("");
+
+      return TP_CONNECTION_PRESENCE_TYPE_AVAILABLE;
+    }
+
   if (status != NULL)
     *status = g_strdup (priv->most_available_status);
 
@@ -1280,7 +1323,7 @@ tp_account_manager_create_account_async (TpAccountManager *manager,
  * object, with the %TP_ACCOUNT_FEATURE_CORE feature ready on it.
  *
  * The caller must keep a ref to the returned object using g_object_ref() if
- * it is to be kept.
+ * it is to be kept beyond the lifetime of @result.
  *
  * Returns: (transfer none): a new #TpAccount which was just created on
  *  success, otherwise %NULL
