@@ -64,6 +64,8 @@ TpContactListState tp_base_contact_list_get_state (TpBaseContactList *self,
     GError **error);
 TpBaseConnection *tp_base_contact_list_get_connection (
     TpBaseContactList *self, GError **error);
+gboolean tp_base_contact_list_get_download_at_connection (
+    TpBaseContactList *self);
 
 /* ---- Called by subclasses for ContactList (or both) ---- */
 
@@ -112,6 +114,23 @@ void tp_base_contact_list_dup_states (TpBaseContactList *self,
     TpSubscriptionState *publish,
     gchar **publish_request);
 
+typedef void (*TpBaseContactListAsyncFunc) (
+    TpBaseContactList *self,
+    GAsyncReadyCallback callback,
+    gpointer user_data);
+
+void tp_base_contact_list_download_async (TpBaseContactList *self,
+    GAsyncReadyCallback callback,
+    gpointer user_data);
+
+gboolean tp_base_contact_list_download_finish (TpBaseContactList *self,
+    GAsyncResult *result,
+    GError **error);
+
+typedef gboolean (*TpBaseContactListAsyncFinishFunc) (TpBaseContactList *self,
+    GAsyncResult *result,
+    GError **error);
+
 struct _TpBaseContactListClass {
     GObjectClass parent_class;
 
@@ -119,8 +138,11 @@ struct _TpBaseContactListClass {
     TpBaseContactListDupStatesFunc dup_states;
     TpBaseContactListBooleanFunc get_contact_list_persists;
 
+    TpBaseContactListAsyncFunc download_async;
+    TpBaseContactListAsyncFinishFunc download_finish;
+
     /*<private>*/
-    GCallback _padding[7];
+    GCallback _padding[5];
     TpBaseContactListClassPrivate *priv;
 };
 
@@ -152,10 +174,6 @@ typedef void (*TpBaseContactListActOnContactsFunc) (
     GAsyncReadyCallback callback,
     gpointer user_data);
 
-typedef gboolean (*TpBaseContactListAsyncFinishFunc) (TpBaseContactList *self,
-    GAsyncResult *result,
-    GError **error);
-
 struct _TpMutableContactListInterface {
     GTypeInterface parent;
 
@@ -286,6 +304,17 @@ gboolean tp_base_contact_list_block_contacts_finish (TpBaseContactList *self,
     GAsyncResult *result,
     GError **error);
 
+void tp_base_contact_list_block_contacts_with_abuse_async (
+    TpBaseContactList *self,
+    TpHandleSet *contacts,
+    gboolean report_abusive,
+    GAsyncReadyCallback callback,
+    gpointer user_data);
+gboolean tp_base_contact_list_block_contacts_with_abuse_finish (
+    TpBaseContactList *self,
+    GAsyncResult *result,
+    GError **error);
+
 void tp_base_contact_list_unblock_contacts_async (TpBaseContactList *self,
     TpHandleSet *contacts,
     GAsyncReadyCallback callback,
@@ -295,6 +324,13 @@ gboolean tp_base_contact_list_unblock_contacts_finish (TpBaseContactList *self,
     GAsyncResult *result,
     GError **error);
 
+typedef void (*TpBaseContactListBlockContactsWithAbuseFunc) (
+    TpBaseContactList *self,
+    TpHandleSet *contacts,
+    gboolean report_abusive,
+    GAsyncReadyCallback callback,
+    gpointer user_data);
+
 struct _TpBlockableContactListInterface {
     GTypeInterface parent;
 
@@ -302,7 +338,9 @@ struct _TpBlockableContactListInterface {
 
     TpBaseContactListDupContactsFunc dup_blocked_contacts;
 
-    /* _async mandatory-to-implement, _finish has a default implementation
+    /* unblock_contacts_async is mandatory to implement; either
+     * block_contacts_async or block_contacts_with_abuse_async (but not both!)
+     * must also be implemented. _finish have default implementations
      * suitable for a GSimpleAsyncResult */
 
     TpBaseContactListActOnContactsFunc block_contacts_async;
@@ -311,8 +349,11 @@ struct _TpBlockableContactListInterface {
     TpBaseContactListAsyncFinishFunc unblock_contacts_finish;
 
     /* optional to implement */
-
     TpBaseContactListBooleanFunc can_block;
+
+    /* see above. block_contacts_finish is the corresponding _finish function.
+     */
+    TpBaseContactListBlockContactsWithAbuseFunc block_contacts_with_abuse_async;
 };
 
 /* ---- Called by subclasses for ContactGroups ---- */
@@ -540,9 +581,10 @@ void tp_base_contact_list_mixin_register_with_contacts_mixin (
     TpBaseConnection *conn);
 void tp_base_contact_list_mixin_list_iface_init (
     TpSvcConnectionInterfaceContactListClass *klass);
-
 void tp_base_contact_list_mixin_groups_iface_init (
     TpSvcConnectionInterfaceContactGroupsClass *klass);
+void tp_base_contact_list_mixin_blocking_iface_init (
+    TpSvcConnectionInterfaceContactBlockingClass *klass);
 
 G_END_DECLS
 
