@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
+
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/dbus-internal.h>
 
@@ -30,8 +32,6 @@
 #include <telepathy-glib/proxy-subclass.h>
 #include <telepathy-glib/util.h>
 
-#include "telepathy-glib/_gen/signals-marshal.h"
-
 #include "telepathy-glib/_gen/tp-cli-dbus-daemon-body.h"
 
 #define DEBUG_FLAG TP_DEBUG_PROXY
