@@ -1,6 +1,6 @@
 /* TpProtocol
  *
- * Copyright © 2010 Collabora Ltd.
+ * Copyright © 2010-2012 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,8 @@
  * Since: 0.11.11
  */
 
+#include "config.h"
+
 #include <telepathy-glib/protocol.h>
 #include <telepathy-glib/protocol-internal.h>
 
@@ -43,7 +45,6 @@
 #include "telepathy-glib/debug-internal.h"
 #include "telepathy-glib/proxy-internal.h"
 
-#include "telepathy-glib/_gen/signals-marshal.h"
 #include "telepathy-glib/_gen/tp-cli-protocol-body.h"
 
 #include <string.h>
@@ -876,6 +877,7 @@ tp_protocol_has_param (TpProtocol *self,
   return (tp_protocol_get_param (self, param) != NULL);
 }
 
+/* FIXME: in Telepathy 1.0, rename to tp_protocol_borrow_param or remove */
 /**
  * tp_protocol_get_param:
  * @self: a protocol
@@ -896,6 +898,30 @@ const TpConnectionManagerParam *tp_protocol_get_param (TpProtocol *self,
       &self->priv->protocol_struct, param);
 }
 
+/* FIXME: in Telepathy 1.0, rename to tp_protocol_get_param */
+/**
+ * tp_protocol_dup_param:
+ * @self: a protocol
+ * @param: a parameter name
+ *
+ * <!-- no more to say -->
+ *
+ * Returns: (transfer full): a structure representing the parameter @param,
+ *  or %NULL if not supported. Free with tp_connection_manager_param_free()
+ *
+ * Since: 0.17.6
+ */
+TpConnectionManagerParam *
+tp_protocol_dup_param (TpProtocol *self,
+    const gchar *param)
+{
+  g_return_val_if_fail (TP_IS_PROTOCOL (self), NULL);
+
+  return tp_connection_manager_param_copy (
+      tp_connection_manager_protocol_get_param (
+        &self->priv->protocol_struct, param));
+}
+
 /**
  * tp_protocol_can_register:
  * @self: a protocol
@@ -935,6 +961,60 @@ tp_protocol_dup_param_names (TpProtocol *self)
       &self->priv->protocol_struct);
 }
 
+/**
+ * tp_protocol_borrow_params: (skip)
+ * @self: a protocol
+ *
+ * Returns an array of parameters supported by this connection manager,
+ * without additional memory allocations. The returned array is owned by
+ * @self, and must not be used after @self has been freed.
+ *
+ * Returns: (transfer none): an array of #TpConnectionManagerParam structures,
+ *  terminated by one whose @name is %NULL
+ *
+ * Since: 0.17.6
+ */
+const TpConnectionManagerParam *
+tp_protocol_borrow_params (TpProtocol *self)
+{
+  g_return_val_if_fail (TP_IS_PROTOCOL (self), NULL);
+
+  return self->priv->protocol_struct.params;
+}
+
+/**
+ * tp_protocol_dup_params:
+ * @self: a protocol
+ *
+ * Returns a list of parameters supported by this connection manager.
+ *
+ * The returned list must be freed by the caller, for instance with
+ * <literal>g_list_free_full (l,
+ * (GDestroyNotify) tp_connection_manager_param_free)</literal>.
+ *
+ * Returns: (transfer full) (element-type TelepathyGLib.ConnectionManagerParam):
+ *  a list of #TpConnectionManagerParam structures, owned by the caller
+ *
+ * Since: 0.17.6
+ */
+GList *
+tp_protocol_dup_params (TpProtocol *self)
+{
+  guint i;
+  GList *ret = NULL;
+
+  g_return_val_if_fail (TP_IS_PROTOCOL (self), NULL);
+
+  for (i = 0; self->priv->protocol_struct.params[i].name != NULL; i++)
+    {
+      ret = g_list_prepend (ret,
+          tp_connection_manager_param_copy (
+            &(self->priv->protocol_struct.params[i])));
+    }
+
+  return g_list_reverse (ret);
+}
+
 /**
  * tp_protocol_get_vcard_field:
  * @self: a protocol object
@@ -1080,6 +1160,10 @@ init_gvalue_from_dbus_sig (const gchar *sig,
           g_value_init (value, G_TYPE_STRV);
           return TRUE;
 
+        case 'o':
+          g_value_init (value, TP_ARRAY_TYPE_OBJECT_PATH_LIST);
+          return TRUE;
+
         case 'y':
           g_value_init (value, DBUS_TYPE_G_UCHAR_ARRAY);
           return TRUE;
@@ -1259,6 +1343,43 @@ parse_default_value (GValue *value,
 
               return TRUE;
             }
+
+        case 'o':
+            {
+              gsize len = 0;
+              GStrv strv = g_key_file_get_string_list (file, group, key, &len,
+                  &error);
+              gchar **iter;
+              GPtrArray *arr;
+
+              if (error != NULL)
+                {
+                  g_error_free (error);
+                  return FALSE;
+                }
+
+              for (iter = strv; iter != NULL && *iter != NULL; iter++)
+                {
+                  if (!g_variant_is_object_path (*iter))
+                    {
+                      g_strfreev (strv);
+                      return FALSE;
+                    }
+                }
+
+              arr = g_ptr_array_sized_new (len);
+
+              for (iter = strv; iter != NULL && *iter != NULL; iter++)
+                {
+                  /* transfer ownership */
+                  g_ptr_array_add (arr, *iter);
+                }
+
+              g_free (strv);
+              g_value_take_boxed (value, arr);
+
+              return TRUE;
+            }
         }
     }
 
