@@ -25,6 +25,7 @@
 #include <glib-object.h>
 
 #include <telepathy-glib/base-client.h>
+#include <telepathy-glib/defs.h>
 
 G_BEGIN_DECLS
 
@@ -79,7 +80,9 @@ TpBaseClient * tp_simple_handler_new (TpDBusDaemon *dbus,
     gboolean uniquify,
     TpSimpleHandlerHandleChannelsImpl callback,
     gpointer user_data,
-    GDestroyNotify destroy);
+    GDestroyNotify destroy)
+    _TP_GNUC_DEPRECATED_FOR (tp_simple_handler_new_with_factory);
+
 
 TpBaseClient *tp_simple_handler_new_with_am (TpAccountManager *account_manager,
     gboolean bypass_approval,
@@ -90,6 +93,16 @@ TpBaseClient *tp_simple_handler_new_with_am (TpAccountManager *account_manager,
     gpointer user_data,
     GDestroyNotify destroy);
 
+TpBaseClient *tp_simple_handler_new_with_factory (
+    TpSimpleClientFactory *factory,
+    gboolean bypass_approval,
+    gboolean requests,
+    const gchar *name,
+    gboolean uniquify,
+    TpSimpleHandlerHandleChannelsImpl callback,
+    gpointer user_data,
+    GDestroyNotify destroy);
+
 G_END_DECLS
 
 #endif
