@@ -8,17 +8,27 @@
  * notice and this notice are preserved.
  */
 
+#include "config.h"
+
 #include <telepathy-glib/telepathy-glib.h>
 
 #include "tests/lib/echo-cm.h"
 #include "tests/lib/util.h"
 
+typedef enum {
+    ACTIVATE_CM = (1 << 0),
+    USE_CWR = (1 << 1),
+    USE_OLD_LIST = (1 << 2)
+} TestFlags;
+
 typedef struct {
     GMainLoop *mainloop;
     TpDBusDaemon *dbus;
     TpTestsEchoConnectionManager *service_cm;
 
     TpConnectionManager *cm;
+    TpConnectionManager *echo;
+    TpConnectionManager *spurious;
     GError *error /* initialized where needed */;
 } Test;
 
@@ -116,10 +126,11 @@ static void
 teardown (Test *test,
           gconstpointer data)
 {
-  g_object_unref (test->service_cm);
-  test->service_cm = NULL;
-  g_object_unref (test->dbus);
-  test->dbus = NULL;
+  g_clear_object (&test->service_cm);
+  g_clear_object (&test->dbus);
+  g_clear_object (&test->cm);
+  g_clear_object (&test->echo);
+  g_clear_object (&test->spurious);
   g_main_loop_unref (test->mainloop);
   test->mainloop = NULL;
 }
@@ -166,8 +177,7 @@ test_nothing_got_info (Test *test,
   test->cm = tp_connection_manager_new (test->dbus, "not_actually_there",
       NULL, &error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (error);
 
   /* Spin the mainloop until we get the got-info signal. This API is rubbish,
    * but it's better than it used to be... */
@@ -181,6 +191,7 @@ test_nothing_got_info (Test *test,
   g_assert_cmpuint (test->cm->running, ==, FALSE);
   g_assert_cmpuint (test->cm->info_source, ==, TP_CM_INFO_SOURCE_NONE);
   g_assert (test->cm->protocols == NULL);
+  g_assert (tp_connection_manager_dup_protocols (test->cm) == NULL);
 }
 
 static void
@@ -212,8 +223,7 @@ test_file_got_info (Test *test,
   test->cm = tp_connection_manager_new (test->dbus, "spurious",
       NULL, &error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (error);
 
   g_test_bug ("18207");
   id = g_signal_connect (test->cm, "got-info",
@@ -348,12 +358,12 @@ test_complex_file_got_info (Test *test,
   const TpConnectionManagerParam *param;
   const TpConnectionManagerProtocol *protocol;
   gchar **strv;
+  GPtrArray *arr;
 
   test->cm = tp_connection_manager_new (test->dbus, "test_manager_file",
       NULL, &error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (error);
 
   g_test_bug ("18207");
   id = g_signal_connect (test->cm, "got-info",
@@ -668,6 +678,17 @@ test_complex_file_got_info (Test *test,
   g_assert_cmpint (g_value_get_uchar (&param->default_value), ==, 42);
 
   param = &protocol->params[22];
+  g_assert_cmpstr (param->name, ==, "ao");
+  g_assert_cmpuint (param->flags, ==, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT);
+  g_assert_cmpstr (param->dbus_signature, ==, "ao");
+  g_assert (G_VALUE_HOLDS (&param->default_value,
+        TP_ARRAY_TYPE_OBJECT_PATH_LIST));
+  arr = g_value_get_boxed (&param->default_value);
+  g_assert_cmpuint (arr->len, ==, 2);
+  g_assert_cmpstr ((gchar *) g_ptr_array_index (arr, 0), ==, "/misc");
+  g_assert_cmpstr ((gchar *) g_ptr_array_index (arr, 1), ==, "/other");
+
+  param = &protocol->params[23];
   g_assert (param->name == NULL);
 }
 
@@ -696,8 +717,7 @@ test_dbus_got_info (Test *test,
       TP_BASE_CONNECTION_MANAGER_GET_CLASS (test->service_cm)->cm_dbus_name,
       NULL, &error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (error);
 
   g_test_bug ("18207");
   id = g_signal_connect (test->cm, "got-info",
@@ -722,29 +742,42 @@ ready_or_not (TpConnectionManager *self,
 
 static void
 test_nothing_ready (Test *test,
-                    gconstpointer data G_GNUC_UNUSED)
+                    gconstpointer data)
 {
   gchar *name;
   guint info_source;
+  TestFlags flags = GPOINTER_TO_INT (data);
 
   test->error = NULL;
   test->cm = tp_connection_manager_new (test->dbus, "nonexistent_cm",
       NULL, &test->error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (test->error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (test->error);
 
   g_test_bug ("18291");
 
-  tp_connection_manager_call_when_ready (test->cm, ready_or_not,
-      test, NULL, NULL);
-  g_main_loop_run (test->mainloop);
-  g_assert (test->error != NULL);
-  g_clear_error (&test->error);
+  if (flags & USE_CWR)
+    {
+      tp_connection_manager_call_when_ready (test->cm, ready_or_not,
+          test, NULL, NULL);
+      g_main_loop_run (test->mainloop);
+      g_assert (test->error != NULL);
+      g_clear_error (&test->error);
+    }
+  else
+    {
+      tp_tests_proxy_run_until_prepared_or_failed (test->cm, NULL,
+          &test->error);
+
+      g_assert_error (test->error, DBUS_GERROR, DBUS_GERROR_SERVICE_UNKNOWN);
+    }
 
   g_assert_cmpstr (tp_connection_manager_get_name (test->cm), ==,
       "nonexistent_cm");
   g_assert_cmpuint (tp_connection_manager_is_ready (test->cm), ==, FALSE);
+  g_assert_cmpuint (tp_proxy_is_prepared (test->cm,
+        TP_CONNECTION_MANAGER_FEATURE_CORE), ==, FALSE);
+  g_assert (tp_proxy_get_invalidated (test->cm) == NULL);
   g_assert_cmpuint (tp_connection_manager_is_running (test->cm), ==, FALSE);
   g_assert_cmpuint (tp_connection_manager_get_info_source (test->cm), ==,
       TP_CM_INFO_SOURCE_NONE);
@@ -760,29 +793,40 @@ test_nothing_ready (Test *test,
 
 static void
 test_file_ready (Test *test,
-                 gconstpointer data G_GNUC_UNUSED)
+                 gconstpointer data)
 {
   gchar *name;
   guint info_source;
+  TestFlags flags = GPOINTER_TO_INT (data);
+  GList *l;
 
   test->error = NULL;
   test->cm = tp_connection_manager_new (test->dbus, "spurious",
       NULL, &test->error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (test->error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (test->error);
 
   g_test_bug ("18291");
 
-  tp_connection_manager_call_when_ready (test->cm, ready_or_not,
-      test, NULL, NULL);
-  g_main_loop_run (test->mainloop);
-  g_assert (test->error == NULL);
+  if (flags & USE_CWR)
+    {
+      tp_connection_manager_call_when_ready (test->cm, ready_or_not,
+          test, NULL, NULL);
+      g_main_loop_run (test->mainloop);
+      g_assert_no_error (test->error);
+    }
+  else
+    {
+      tp_tests_proxy_run_until_prepared (test->cm, NULL);
+    }
 
   g_assert_cmpstr (tp_connection_manager_get_name (test->cm), ==,
       "spurious");
   g_assert_cmpuint (tp_connection_manager_is_ready (test->cm), ==, TRUE);
   g_assert_cmpuint (tp_connection_manager_is_running (test->cm), ==, FALSE);
+  g_assert_cmpuint (tp_proxy_is_prepared (test->cm,
+        TP_CONNECTION_MANAGER_FEATURE_CORE), ==, TRUE);
+  g_assert (tp_proxy_get_invalidated (test->cm) == NULL);
   g_assert_cmpuint (tp_connection_manager_get_info_source (test->cm), ==,
       TP_CM_INFO_SOURCE_FILE);
 
@@ -794,31 +838,46 @@ test_file_ready (Test *test,
   g_assert_cmpuint (info_source, ==, TP_CM_INFO_SOURCE_FILE);
   g_free (name);
 
+  l = tp_connection_manager_dup_protocols (test->cm);
+  g_assert_cmpuint (g_list_length (l), ==, 2);
+  g_assert (TP_IS_PROTOCOL (l->data));
+  g_assert (TP_IS_PROTOCOL (l->next->data));
+  g_list_free_full (l, g_object_unref);
 }
 
 static void
 test_complex_file_ready (Test *test,
-                         gconstpointer data G_GNUC_UNUSED)
+                         gconstpointer data)
 {
   gchar *name;
   guint info_source;
+  TestFlags flags = GPOINTER_TO_INT (data);
 
   test->error = NULL;
   test->cm = tp_connection_manager_new (test->dbus, "test_manager_file",
       NULL, &test->error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (test->error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (test->error);
 
   g_test_bug ("18291");
 
-  tp_connection_manager_call_when_ready (test->cm, ready_or_not,
-      test, NULL, NULL);
-  g_main_loop_run (test->mainloop);
-  g_assert (test->error == NULL);
+  if (flags & USE_CWR)
+    {
+      tp_connection_manager_call_when_ready (test->cm, ready_or_not,
+          test, NULL, NULL);
+      g_main_loop_run (test->mainloop);
+      g_assert_no_error (test->error);
+    }
+  else
+    {
+      tp_tests_proxy_run_until_prepared (test->cm, NULL);
+    }
 
   g_assert_cmpstr (tp_connection_manager_get_name (test->cm), ==,
       "test_manager_file");
+  g_assert_cmpuint (tp_proxy_is_prepared (test->cm,
+        TP_CONNECTION_MANAGER_FEATURE_CORE), ==, TRUE);
+  g_assert (tp_proxy_get_invalidated (test->cm) == NULL);
   g_assert_cmpuint (tp_connection_manager_is_ready (test->cm), ==, TRUE);
   g_assert_cmpuint (tp_connection_manager_is_running (test->cm), ==, FALSE);
   g_assert_cmpuint (tp_connection_manager_get_info_source (test->cm), ==,
@@ -846,17 +905,17 @@ test_dbus_ready (Test *test,
 {
   gchar *name;
   guint info_source;
-  const gboolean activate = GPOINTER_TO_INT (data);
+  const TestFlags flags = GPOINTER_TO_INT (data);
+  GList *l;
 
   test->error = NULL;
   test->cm = tp_connection_manager_new (test->dbus,
       TP_BASE_CONNECTION_MANAGER_GET_CLASS (test->service_cm)->cm_dbus_name,
       NULL, &test->error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (test->error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (test->error);
 
-  if (activate)
+  if (flags & ACTIVATE_CM)
     {
       g_test_bug ("23524");
 
@@ -873,13 +932,23 @@ test_dbus_ready (Test *test,
       g_test_bug ("18291");
     }
 
-  tp_connection_manager_call_when_ready (test->cm, ready_or_not,
-      test, NULL, NULL);
-  g_main_loop_run (test->mainloop);
-  g_assert (test->error == NULL);
+  if (flags & USE_CWR)
+    {
+      tp_connection_manager_call_when_ready (test->cm, ready_or_not,
+          test, NULL, NULL);
+      g_main_loop_run (test->mainloop);
+      g_assert_no_error (test->error);
+    }
+  else
+    {
+      tp_tests_proxy_run_until_prepared (test->cm, NULL);
+    }
 
   g_assert_cmpstr (tp_connection_manager_get_name (test->cm), ==,
       "example_echo");
+  g_assert_cmpuint (tp_proxy_is_prepared (test->cm,
+        TP_CONNECTION_MANAGER_FEATURE_CORE), ==, TRUE);
+  g_assert (tp_proxy_get_invalidated (test->cm) == NULL);
   g_assert_cmpuint (tp_connection_manager_is_ready (test->cm), ==, TRUE);
   g_assert_cmpuint (tp_connection_manager_is_running (test->cm), ==, TRUE);
   g_assert_cmpuint (tp_connection_manager_get_info_source (test->cm), ==,
@@ -892,6 +961,11 @@ test_dbus_ready (Test *test,
   g_assert_cmpstr (name, ==, "example_echo");
   g_assert_cmpuint (info_source, ==, TP_CM_INFO_SOURCE_LIVE);
   g_free (name);
+
+  l = tp_connection_manager_dup_protocols (test->cm);
+  g_assert_cmpuint (g_list_length (l), ==, 1);
+  g_assert_cmpstr (tp_protocol_get_name (l->data), ==, "example");
+  g_list_free_full (l, g_object_unref);
 }
 
 static void
@@ -900,7 +974,7 @@ test_dbus_fallback (Test *test,
 {
   gchar *name;
   guint info_source;
-  const gboolean activate = GPOINTER_TO_INT (data);
+  const TestFlags flags = GPOINTER_TO_INT (data);
   TpBaseConnectionManager *service_cm_as_base;
   gboolean ok;
 
@@ -922,10 +996,9 @@ test_dbus_fallback (Test *test,
       TP_BASE_CONNECTION_MANAGER_GET_CLASS (test->service_cm)->cm_dbus_name,
       NULL, &test->error);
   g_assert (TP_IS_CONNECTION_MANAGER (test->cm));
-  g_assert (test->error == NULL);
-  g_test_queue_unref (test->cm);
+  g_assert_no_error (test->error);
 
-  if (activate)
+  if (flags & ACTIVATE_CM)
     {
       g_test_bug ("23524");
 
@@ -942,13 +1015,23 @@ test_dbus_fallback (Test *test,
       g_test_bug ("18291");
     }
 
-  tp_connection_manager_call_when_ready (test->cm, ready_or_not,
-      test, NULL, NULL);
-  g_main_loop_run (test->mainloop);
-  g_assert (test->error == NULL);
+  if (flags & USE_CWR)
+    {
+      tp_connection_manager_call_when_ready (test->cm, ready_or_not,
+          test, NULL, NULL);
+      g_main_loop_run (test->mainloop);
+      g_assert_no_error (test->error);
+    }
+  else
+    {
+      tp_tests_proxy_run_until_prepared (test->cm, NULL);
+    }
 
   g_assert_cmpstr (tp_connection_manager_get_name (test->cm), ==,
       "example_echo");
+  g_assert_cmpuint (tp_proxy_is_prepared (test->cm,
+        TP_CONNECTION_MANAGER_FEATURE_CORE), ==, TRUE);
+  g_assert (tp_proxy_get_invalidated (test->cm) == NULL);
   g_assert_cmpuint (tp_connection_manager_is_ready (test->cm), ==, TRUE);
   g_assert_cmpuint (tp_connection_manager_is_running (test->cm), ==, TRUE);
   g_assert_cmpuint (tp_connection_manager_get_info_source (test->cm), ==,
@@ -971,46 +1054,85 @@ on_listed_connection_managers (TpConnectionManager * const * cms,
                                GObject *weak_object G_GNUC_UNUSED)
 {
   Test *test = user_data;
-  TpConnectionManager *spurious;
-  TpConnectionManager *echo;
 
   g_assert_cmpuint ((guint) n_cms, ==, 2);
   g_assert (cms[2] == NULL);
 
   if (tp_connection_manager_is_running (cms[0]))
     {
-      echo = cms[0];
-      spurious = cms[1];
+      test->echo = g_object_ref (cms[0]);
+      test->spurious = g_object_ref (cms[1]);
     }
   else
     {
-      spurious = cms[0];
-      echo = cms[1];
+      test->spurious = g_object_ref (cms[0]);
+      test->echo = g_object_ref (cms[1]);
     }
 
-  g_assert (tp_connection_manager_is_running (echo));
-  g_assert (!tp_connection_manager_is_running (spurious));
-
-  g_assert (tp_connection_manager_is_ready (echo));
-  g_assert (tp_connection_manager_is_ready (spurious));
-
-  g_assert_cmpuint (tp_connection_manager_get_info_source (echo),
-      ==, TP_CM_INFO_SOURCE_LIVE);
-  g_assert_cmpuint (tp_connection_manager_get_info_source (spurious),
-      ==, TP_CM_INFO_SOURCE_FILE);
-
-  g_assert (tp_connection_manager_has_protocol (echo, "example"));
-  g_assert (tp_connection_manager_has_protocol (spurious, "normal"));
-
   g_main_loop_quit (test->mainloop);
 }
 
 static void
 test_list (Test *test,
-           gconstpointer data G_GNUC_UNUSED)
+           gconstpointer data)
 {
-  tp_list_connection_managers (test->dbus, on_listed_connection_managers,
-      test, NULL, NULL);
+  TestFlags flags = GPOINTER_TO_INT (data);
+
+  if (flags & USE_OLD_LIST)
+    {
+      tp_list_connection_managers (test->dbus, on_listed_connection_managers,
+          test, NULL, NULL);
+      g_main_loop_run (test->mainloop);
+    }
+  else
+    {
+      GAsyncResult *res = NULL;
+      GList *cms;
+
+      tp_list_connection_managers_async (test->dbus, tp_tests_result_ready_cb,
+          &res);
+      tp_tests_run_until_result (&res);
+      cms = tp_list_connection_managers_finish (res, &test->error);
+      g_assert_no_error (test->error);
+      g_assert_cmpuint (g_list_length (cms), ==, 2);
+
+      /* transfer ownership */
+      if (tp_connection_manager_is_running (cms->data))
+        {
+          test->echo = cms->data;
+          test->spurious = cms->next->data;
+        }
+      else
+        {
+          test->spurious = cms->data;
+          test->echo = cms->next->data;
+        }
+
+      g_object_unref (res);
+      g_list_free (cms);
+    }
+
+  g_assert (tp_connection_manager_is_running (test->echo));
+  g_assert (!tp_connection_manager_is_running (test->spurious));
+
+  g_assert (tp_proxy_is_prepared (test->echo,
+        TP_CONNECTION_MANAGER_FEATURE_CORE));
+  g_assert (tp_proxy_is_prepared (test->spurious,
+        TP_CONNECTION_MANAGER_FEATURE_CORE));
+
+  g_assert (tp_proxy_get_invalidated (test->echo) == NULL);
+  g_assert (tp_proxy_get_invalidated (test->spurious) == NULL);
+
+  g_assert (tp_connection_manager_is_ready (test->echo));
+  g_assert (tp_connection_manager_is_ready (test->spurious));
+
+  g_assert_cmpuint (tp_connection_manager_get_info_source (test->echo),
+      ==, TP_CM_INFO_SOURCE_LIVE);
+  g_assert_cmpuint (tp_connection_manager_get_info_source (test->spurious),
+      ==, TP_CM_INFO_SOURCE_FILE);
+
+  g_assert (tp_connection_manager_has_protocol (test->echo, "example"));
+  g_assert (tp_connection_manager_has_protocol (test->spurious, "normal"));
 }
 
 int
@@ -1031,21 +1153,42 @@ main (int argc,
   g_test_add ("/cm/dbus (old)", Test, NULL, setup, test_dbus_got_info,
       teardown);
 
-  g_test_add ("/cm/nothing", Test, NULL, setup, test_nothing_ready,
-      teardown);
-  g_test_add ("/cm/file", Test, NULL, setup, test_file_ready, teardown);
-  g_test_add ("/cm/file (complex)", Test, NULL, setup,
+  g_test_add ("/cm/nothing", Test, GINT_TO_POINTER (0),
+      setup, test_nothing_ready, teardown);
+  g_test_add ("/cm/nothing/cwr", Test, GINT_TO_POINTER (USE_CWR),
+      setup, test_nothing_ready, teardown);
+  g_test_add ("/cm/file", Test, GINT_TO_POINTER (0),
+      setup, test_file_ready, teardown);
+  g_test_add ("/cm/file/cwr", Test, GINT_TO_POINTER (USE_CWR),
+      setup, test_file_ready, teardown);
+  g_test_add ("/cm/file/complex", Test, GINT_TO_POINTER (0), setup,
       test_complex_file_ready, teardown);
-  g_test_add ("/cm/dbus", Test, GINT_TO_POINTER (FALSE), setup,
+  g_test_add ("/cm/file/complex/cwr", Test, GINT_TO_POINTER (USE_CWR), setup,
+      test_complex_file_ready, teardown);
+  g_test_add ("/cm/dbus", Test, GINT_TO_POINTER (0), setup,
       test_dbus_ready, teardown);
-  g_test_add ("/cm/dbus-and-activate", Test, GINT_TO_POINTER (TRUE), setup,
+  g_test_add ("/cm/dbus/cwr", Test, GINT_TO_POINTER (USE_CWR), setup,
       test_dbus_ready, teardown);
-  g_test_add ("/cm/dbus-fallback", Test, GINT_TO_POINTER (FALSE), setup,
+  g_test_add ("/cm/dbus/activate", Test, GINT_TO_POINTER (ACTIVATE_CM),
+      setup, test_dbus_ready, teardown);
+  g_test_add ("/cm/dbus/activate/cwr", Test,
+      GINT_TO_POINTER (USE_CWR|ACTIVATE_CM),
+      setup, test_dbus_ready, teardown);
+  g_test_add ("/cm/dbus-fallback", Test, GINT_TO_POINTER (0), setup,
       test_dbus_fallback, teardown);
-  g_test_add ("/cm/dbus-fallback-activate", Test, GINT_TO_POINTER (TRUE),
+  g_test_add ("/cm/dbus-fallback/cwr", Test, GINT_TO_POINTER (USE_CWR), setup,
+      test_dbus_fallback, teardown);
+  g_test_add ("/cm/dbus-fallback/activate", Test,
+      GINT_TO_POINTER (ACTIVATE_CM),
+      setup, test_dbus_fallback, teardown);
+  g_test_add ("/cm/dbus-fallback/activate/cwr", Test,
+      GINT_TO_POINTER (ACTIVATE_CM | USE_CWR),
       setup, test_dbus_fallback, teardown);
 
-  g_test_add ("/cm/list", Test, NULL, setup, test_list, teardown);
+  g_test_add ("/cm/list", Test, GINT_TO_POINTER (0),
+      setup, test_list, teardown);
+  g_test_add ("/cm/list", Test, GINT_TO_POINTER (USE_OLD_LIST),
+      setup, test_list, teardown);
 
   return g_test_run ();
 }
