@@ -16,7 +16,7 @@
 #include <string.h>
 #include <telepathy-glib/telepathy-glib.h>
 
-struct _TestContactListManagerPrivate
+struct _TpTestsContactListManagerPrivate
 {
   TpBaseConnection *conn;
 
@@ -33,13 +33,17 @@ struct _TestContactListManagerPrivate
 static void contact_groups_iface_init (TpContactGroupListInterface *iface);
 static void mutable_contact_groups_iface_init (
     TpMutableContactGroupListInterface *iface);
+static void mutable_iface_init (
+    TpMutableContactListInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (TestContactListManager, test_contact_list_manager,
+G_DEFINE_TYPE_WITH_CODE (TpTestsContactListManager, tp_tests_contact_list_manager,
     TP_TYPE_BASE_CONTACT_LIST,
     G_IMPLEMENT_INTERFACE (TP_TYPE_CONTACT_GROUP_LIST,
       contact_groups_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_MUTABLE_CONTACT_GROUP_LIST,
-      mutable_contact_groups_iface_init))
+      mutable_contact_groups_iface_init)
+    G_IMPLEMENT_INTERFACE (TP_TYPE_MUTABLE_CONTACT_LIST,
+      mutable_iface_init))
 
 typedef struct {
   TpSubscriptionState subscribe;
@@ -64,7 +68,7 @@ contact_detail_destroy (gpointer p)
 }
 
 static ContactDetails *
-lookup_contact (TestContactListManager *self,
+lookup_contact (TpTestsContactListManager *self,
                 TpHandle handle)
 {
   return g_hash_table_lookup (self->priv->contact_details,
@@ -72,7 +76,7 @@ lookup_contact (TestContactListManager *self,
 }
 
 static ContactDetails *
-ensure_contact (TestContactListManager *self,
+ensure_contact (TpTestsContactListManager *self,
                 TpHandle handle)
 {
   ContactDetails *d = lookup_contact (self, handle);
@@ -96,17 +100,17 @@ ensure_contact (TestContactListManager *self,
 }
 
 static void
-test_contact_list_manager_init (TestContactListManager *self)
+tp_tests_contact_list_manager_init (TpTestsContactListManager *self)
 {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-      TEST_TYPE_CONTACT_LIST_MANAGER, TestContactListManagerPrivate);
+      TP_TESTS_TYPE_CONTACT_LIST_MANAGER, TpTestsContactListManagerPrivate);
 
   self->priv->contact_details = g_hash_table_new_full (g_direct_hash,
       g_direct_equal, NULL, contact_detail_destroy);
 }
 
 static void
-close_all (TestContactListManager *self)
+close_all (TpTestsContactListManager *self)
 {
   if (self->priv->status_changed_id != 0)
     {
@@ -121,18 +125,18 @@ close_all (TestContactListManager *self)
 static void
 dispose (GObject *object)
 {
-  TestContactListManager *self = TEST_CONTACT_LIST_MANAGER (object);
+  TpTestsContactListManager *self = TP_TESTS_CONTACT_LIST_MANAGER (object);
 
   close_all (self);
 
-  ((GObjectClass *) test_contact_list_manager_parent_class)->dispose (
+  ((GObjectClass *) tp_tests_contact_list_manager_parent_class)->dispose (
     object);
 }
 
 static TpHandleSet *
 contact_list_dup_contacts (TpBaseContactList *base)
 {
-  TestContactListManager *self = TEST_CONTACT_LIST_MANAGER (base);
+  TpTestsContactListManager *self = TP_TESTS_CONTACT_LIST_MANAGER (base);
   TpHandleSet *set;
   GHashTableIter iter;
   gpointer k, v;
@@ -160,7 +164,7 @@ contact_list_dup_states (TpBaseContactList *base,
     TpSubscriptionState *publish,
     gchar **publish_request)
 {
-  TestContactListManager *self = TEST_CONTACT_LIST_MANAGER (base);
+  TpTestsContactListManager *self = TP_TESTS_CONTACT_LIST_MANAGER (base);
   ContactDetails *d = lookup_contact (self, contact);
 
   if (d == NULL)
@@ -190,7 +194,7 @@ contact_list_dup_states (TpBaseContactList *base,
 static GStrv
 contact_list_dup_groups (TpBaseContactList *base)
 {
-  TestContactListManager *self = TEST_CONTACT_LIST_MANAGER (base);
+  TpTestsContactListManager *self = TP_TESTS_CONTACT_LIST_MANAGER (base);
   GPtrArray *ret;
 
   if (self->priv->groups != NULL)
@@ -221,7 +225,7 @@ static GStrv
 contact_list_dup_contact_groups (TpBaseContactList *base,
     TpHandle contact)
 {
-  TestContactListManager *self = TEST_CONTACT_LIST_MANAGER (base);
+  TpTestsContactListManager *self = TP_TESTS_CONTACT_LIST_MANAGER (base);
   ContactDetails *d = lookup_contact (self, contact);
   GPtrArray *ret;
 
@@ -253,7 +257,7 @@ static TpHandleSet *
 contact_list_dup_group_members (TpBaseContactList *base,
     const gchar *group)
 {
-  TestContactListManager *self = TEST_CONTACT_LIST_MANAGER (base);
+  TpTestsContactListManager *self = TP_TESTS_CONTACT_LIST_MANAGER (base);
   TpHandleSet *set;
   TpHandle group_handle;
   GHashTableIter iter;
@@ -288,21 +292,35 @@ contact_list_set_contact_groups_async (TpBaseContactList *base,
     GAsyncReadyCallback callback,
     gpointer user_data)
 {
-  TestContactListManager *self = TEST_CONTACT_LIST_MANAGER (base);
+  TpTestsContactListManager *self = TP_TESTS_CONTACT_LIST_MANAGER (base);
   ContactDetails *d;
   TpIntset *set, *added_set, *removed_set;
   GPtrArray *added_names, *removed_names;
+  GPtrArray *new_groups;
   TpIntSetFastIter iter;
   TpHandle group_handle;
   guint i;
 
   d = ensure_contact (self, contact);
 
+  new_groups = g_ptr_array_new ();
   set = tp_intset_new ();
   for (i = 0; i < n; i++)
     {
       group_handle = tp_handle_ensure (self->priv->group_repo, names[i], NULL, NULL);
       tp_intset_add (set, group_handle);
+
+      if (!tp_handle_set_is_member (self->priv->groups, group_handle))
+        {
+          tp_handle_set_add (self->priv->groups, group_handle);
+          g_ptr_array_add (new_groups, (gchar *) names[i]);
+        }
+    }
+
+  if (new_groups->len > 0)
+    {
+      tp_base_contact_list_groups_created ((TpBaseContactList *) self,
+          (const gchar * const *) new_groups->pdata, new_groups->len);
     }
 
   added_set = tp_intset_difference (set, tp_handle_set_peek (d->groups));
@@ -329,15 +347,19 @@ contact_list_set_contact_groups_async (TpBaseContactList *base,
   d->groups = tp_handle_set_new_from_intset (self->priv->group_repo, set);
   tp_intset_destroy (set);
 
-  tp_base_contact_list_one_contact_groups_changed (base, contact,
-      (const gchar * const *) added_names->pdata, added_names->len,
-      (const gchar * const *) removed_names->pdata, removed_names->len);
+  if (added_names->len > 0 || removed_names->len > 0)
+    {
+      tp_base_contact_list_one_contact_groups_changed (base, contact,
+          (const gchar * const *) added_names->pdata, added_names->len,
+          (const gchar * const *) removed_names->pdata, removed_names->len);
+    }
 
   tp_simple_async_report_success_in_idle ((GObject *) self, callback,
       user_data, contact_list_set_contact_groups_async);
 
   g_ptr_array_unref (added_names);
   g_ptr_array_unref (removed_names);
+  g_ptr_array_unref (new_groups);
 }
 
 static void
@@ -399,11 +421,102 @@ contact_list_remove_group_async (TpBaseContactList *base,
   g_object_unref (simple);
 }
 
+static void
+contact_list_request_subscription_async (TpBaseContactList *self,
+    TpHandleSet *contacts,
+    const gchar *message,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GArray *handles;
+
+  handles = tp_handle_set_to_array (contacts);
+  tp_tests_contact_list_manager_request_subscription (
+      (TpTestsContactListManager *) self,
+      handles->len, (TpHandle *) handles->data, message);
+  g_array_unref (handles);
+
+  tp_simple_async_report_success_in_idle ((GObject *) self, callback,
+      user_data, contact_list_request_subscription_async);
+}
+
+static void
+contact_list_authorize_publication_async (TpBaseContactList *self,
+    TpHandleSet *contacts,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GArray *handles;
+
+  handles = tp_handle_set_to_array (contacts);
+  tp_tests_contact_list_manager_authorize_publication (
+      (TpTestsContactListManager *) self,
+      handles->len, (TpHandle *) handles->data);
+  g_array_unref (handles);
+
+  tp_simple_async_report_success_in_idle ((GObject *) self, callback,
+      user_data, contact_list_authorize_publication_async);
+}
+
+static void
+contact_list_remove_contacts_async (TpBaseContactList *self,
+    TpHandleSet *contacts,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GArray *handles;
+
+  handles = tp_handle_set_to_array (contacts);
+  tp_tests_contact_list_manager_remove (
+      (TpTestsContactListManager *) self,
+      handles->len, (TpHandle *) handles->data);
+  g_array_unref (handles);
+
+  tp_simple_async_report_success_in_idle ((GObject *) self, callback,
+      user_data, contact_list_remove_contacts_async);
+}
+
+static void
+contact_list_unsubscribe_async (TpBaseContactList *self,
+    TpHandleSet *contacts,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GArray *handles;
+
+  handles = tp_handle_set_to_array (contacts);
+  tp_tests_contact_list_manager_unsubscribe (
+      (TpTestsContactListManager *) self,
+      handles->len, (TpHandle *) handles->data);
+  g_array_unref (handles);
+
+  tp_simple_async_report_success_in_idle ((GObject *) self, callback,
+      user_data, contact_list_unsubscribe_async);
+}
+
+static void
+contact_list_unpublish_async (TpBaseContactList *self,
+    TpHandleSet *contacts,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GArray *handles;
+
+  handles = tp_handle_set_to_array (contacts);
+  tp_tests_contact_list_manager_unpublish (
+      (TpTestsContactListManager *) self,
+      handles->len, (TpHandle *) handles->data);
+  g_array_unref (handles);
+
+  tp_simple_async_report_success_in_idle ((GObject *) self, callback,
+      user_data, contact_list_unpublish_async);
+}
+
 static void
 status_changed_cb (TpBaseConnection *conn,
                    guint status,
                    guint reason,
-                   TestContactListManager *self)
+                   TpTestsContactListManager *self)
 {
   switch (status)
     {
@@ -424,9 +537,9 @@ status_changed_cb (TpBaseConnection *conn,
 static void
 constructed (GObject *object)
 {
-  TestContactListManager *self = TEST_CONTACT_LIST_MANAGER (object);
+  TpTestsContactListManager *self = TP_TESTS_CONTACT_LIST_MANAGER (object);
   void (*chain_up) (GObject *) =
-      ((GObjectClass *) test_contact_list_manager_parent_class)->constructed;
+      ((GObjectClass *) tp_tests_contact_list_manager_parent_class)->constructed;
 
   if (chain_up != NULL)
     {
@@ -465,12 +578,22 @@ mutable_contact_groups_iface_init (
 }
 
 static void
-test_contact_list_manager_class_init (TestContactListManagerClass *klass)
+mutable_iface_init (TpMutableContactListInterface *iface)
+{
+  iface->request_subscription_async = contact_list_request_subscription_async;
+  iface->authorize_publication_async = contact_list_authorize_publication_async;
+  iface->remove_contacts_async = contact_list_remove_contacts_async;
+  iface->unsubscribe_async = contact_list_unsubscribe_async;
+  iface->unpublish_async = contact_list_unpublish_async;
+}
+
+static void
+tp_tests_contact_list_manager_class_init (TpTestsContactListManagerClass *klass)
 {
   GObjectClass *object_class = (GObjectClass *) klass;
   TpBaseContactListClass *base_class =(TpBaseContactListClass *) klass;
 
-  g_type_class_add_private (klass, sizeof (TestContactListManagerPrivate));
+  g_type_class_add_private (klass, sizeof (TpTestsContactListManagerPrivate));
 
   object_class->constructed = constructed;
   object_class->dispose = dispose;
@@ -480,14 +603,22 @@ test_contact_list_manager_class_init (TestContactListManagerClass *klass)
 }
 
 void
-test_contact_list_manager_add_to_group (TestContactListManager *self,
+tp_tests_contact_list_manager_add_to_group (TpTestsContactListManager *self,
     const gchar *group_name, TpHandle member)
 {
   TpBaseContactList *base = TP_BASE_CONTACT_LIST (self);
   ContactDetails *d = ensure_contact (self, member);
   TpHandle group_handle;
 
-  group_handle = tp_handle_ensure (self->priv->group_repo, group_name, NULL, NULL);
+  group_handle = tp_handle_ensure (self->priv->group_repo,
+      group_name, NULL, NULL);
+
+  if (!tp_handle_set_is_member (self->priv->groups, group_handle))
+    {
+      tp_handle_set_add (self->priv->groups, group_handle);
+      tp_base_contact_list_groups_created ((TpBaseContactList *) self,
+          &group_name, 1);
+    }
 
   tp_handle_set_add (d->groups, group_handle);
   tp_base_contact_list_one_contact_groups_changed (base, member,
@@ -495,7 +626,7 @@ test_contact_list_manager_add_to_group (TestContactListManager *self,
 }
 
 void
-test_contact_list_manager_remove_from_group (TestContactListManager *self,
+tp_tests_contact_list_manager_remove_from_group (TpTestsContactListManager *self,
     const gchar *group_name, TpHandle member)
 {
   TpBaseContactList *base = TP_BASE_CONTACT_LIST (self);
@@ -513,12 +644,12 @@ test_contact_list_manager_remove_from_group (TestContactListManager *self,
 }
 
 typedef struct {
-    TestContactListManager *self;
+    TpTestsContactListManager *self;
     TpHandleSet *handles;
 } SelfAndContact;
 
 static SelfAndContact *
-self_and_contact_new (TestContactListManager *self,
+self_and_contact_new (TpTestsContactListManager *self,
   TpHandleSet *handles)
 {
   SelfAndContact *ret = g_slice_new0 (SelfAndContact);
@@ -600,7 +731,7 @@ receive_unauthorized (gpointer p)
 }
 
 void
-test_contact_list_manager_request_subscription (TestContactListManager *self,
+tp_tests_contact_list_manager_request_subscription (TpTestsContactListManager *self,
     guint n_members, TpHandle *members,  const gchar *message)
 {
   TpHandleSet *handles;
@@ -643,7 +774,7 @@ test_contact_list_manager_request_subscription (TestContactListManager *self,
 }
 
 void
-test_contact_list_manager_unsubscribe (TestContactListManager *self,
+tp_tests_contact_list_manager_unsubscribe (TpTestsContactListManager *self,
     guint n_members, TpHandle *members)
 {
   TpHandleSet *handles;
@@ -668,7 +799,7 @@ test_contact_list_manager_unsubscribe (TestContactListManager *self,
 }
 
 void
-test_contact_list_manager_authorize_publication (TestContactListManager *self,
+tp_tests_contact_list_manager_authorize_publication (TpTestsContactListManager *self,
     guint n_members, TpHandle *members)
 {
   TpHandleSet *handles;
@@ -694,7 +825,7 @@ test_contact_list_manager_authorize_publication (TestContactListManager *self,
 }
 
 void
-test_contact_list_manager_unpublish (TestContactListManager *self,
+tp_tests_contact_list_manager_unpublish (TpTestsContactListManager *self,
     guint n_members, TpHandle *members)
 {
   TpHandleSet *handles;
@@ -720,7 +851,7 @@ test_contact_list_manager_unpublish (TestContactListManager *self,
 }
 
 void
-test_contact_list_manager_remove (TestContactListManager *self,
+tp_tests_contact_list_manager_remove (TpTestsContactListManager *self,
     guint n_members, TpHandle *members)
 {
   TpHandleSet *handles;
@@ -746,7 +877,7 @@ test_contact_list_manager_remove (TestContactListManager *self,
 }
 
 void
-test_contact_list_manager_add_initial_contacts (TestContactListManager *self,
+tp_tests_contact_list_manager_add_initial_contacts (TpTestsContactListManager *self,
     guint n_members, TpHandle *members)
 {
   TpHandleSet *handles;
